/**
 * @file Basic type definitions for vector types.
 * @license
 * Copyright (c) 2020 Felix Kopp <felix.kopp@stud.th-deg.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

import { LineBasicMaterial, Object3D, Vector3, Line, Geometry } from 'three';

/** Represents a single point in 3D space. */
export interface IPoint {
    /** The x (width) coordinate. */
    x: number;
    /** The y (height) coordinate. */
    y: number;
    /** The z (depth) coordinate. */
    z: number;
}

/** A fixed-position, fixed-length 3D vector. */
export interface IVector {
    /** The starting point, relative to the coordinate origin. */
    base: IPoint;
    /** The end point, relative to the starting point. */
    direction: IPoint;
}

export interface IDrawableVector {
    geom: Geometry; /* TODO: Migrate to BufferGeometry */
    line: Line;
}
